#!/usr/bin/env python
# -*- coding: utf_8 -*-

# based on https://github.com/pymodbus-dev/pymodbus/blob/dev/examples/client_sync.py

import logging
import time
import sys
import struct
import pymodbus
from pymodbus.exceptions import ModbusException
from pymodbus.client import ModbusSerialClient
from  pymodbus.file_message import FileRecord

PORT = '/dev/ttyUSB0'
DEVICE_ADDR = 1

logger = logging.getLogger()
logger.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
logger.addHandler(log_handler)

def main():
    try:
        client = ModbusSerialClient(
            port=PORT,
            timeout=0.2,
            # Common optional paramers:
            #    framer=ModbusRtuFramer,
            #    timeout=10,
            #    retries=3,
            #    retry_on_empty=False,
            #    close_comm_on_error=False,
            #    strict=True,
            baudrate=19200,
            parity="E",
        )
        client.connect()
        logger.info("connected")

        x = client.read_holding_registers(address=1, count=1, slave=DEVICE_ADDR)
        if isinstance(x, ModbusException):
            raise x
        logger.info("holding reg 1: %s: %r", x, x.registers)

        scratch_reg = 2
        client.write_register(address=scratch_reg, value=0x12, slave=DEVICE_ADDR)
        x = client.mask_write_register(address=scratch_reg, and_mask=0xf2, or_mask=0x25, slave=DEVICE_ADDR)
        logger.info("mask write: %s:", x)
        x = client.read_holding_registers(address=scratch_reg, slave=DEVICE_ADDR)
        logger.info("read scratch reg: %s: %r", x, x.registers)
        assert x.registers == [0x17]

        x = client.readwrite_registers(read_address=0, read_count=3, write_address=1,
            write_registers=[struct.unpack(">H", b"PI")[0], 103], slave=DEVICE_ADDR)
        logger.info("read/write regs: %s: %r", x, x.registers)

        if False:
            x = client.write_registers(1, struct.unpack(">H", b"RE")[0], slave=DEVICE_ADDR)
            logger.info("write to reset: %s", x)

        x = client.read_file_record([
            # documentation says tuple of: Reference type, File number, Record Number, Record Length
            # but actually an object is expected:
            FileRecord(file_number=0, record_number=0, record_length=7),
            FileRecord(file_number=0, record_number=1, record_length=5),
            FileRecord(file_number=0, record_number=2, record_length=3),
        ], slave=DEVICE_ADDR)
        logger.info("file 0: %s:", x)
        for r in x.records:
            logger.info("  %s: %r", r, r.record_data)

        logger.info("Bootloader state partition:")
        x = client.read_file_record([
            FileRecord(file_number=1, record_number=int(0x6000/2), record_length=20),
        ], slave=DEVICE_ADDR)
        logger.info("file 1 @ bootloader state partition: %s:", x)
        for r in x.records:
            logger.info("  %s: %r", r, r.record_data)

        toggle = True
        while True:
            s = time.time()
            logger.info(client.write_coil(address=0, value=toggle, slave=DEVICE_ADDR))
            toggle = not toggle
            logger.info("took: %s s" % (time.time() - s))

            s = time.time()
            x = client.read_discrete_inputs(address=0, count=5, slave=DEVICE_ADDR)
            logger.info("read_discrete_inputs: %s: %r", x, x.bits)
            logger.info("took: %s s" % (time.time() - s))

            s = time.time()
            x = client.read_input_registers(address=0, count=25, slave=DEVICE_ADDR)
            logger.info("read_input_registers: %s: %r", x, x.registers)
            logger.info("took: %s s" % (time.time() - s))

            time.sleep(0.1)

    except ModbusException as exc:
        logger.exception("%s", exc)

if __name__ == "__main__":
    main()
